#!/tvbin/tivosh
#
# noppv.tcl - Remove PPV flag from all current showings   (v1.2)
#
# Pretty self explanitory. Just run it. If you wish to see show times in your timezonre
# just add your GMT offset to the command line. IE: "./noppv.tcl -5". Default timezone
# is central (GMT -6).
#
# If you decide to use this script, you are doing so AT YOUR OWN RISK. I have tested this
# script for functionality many times. This doesn't mean it will always work on your machine.
#
# DO NOT USE this if you want to be able to buy PPV. This includes people using a legit sub
# and people using activation. IT WONT WORK. Only use this with a 3M or an emulator.
#

tvsource $tcl_library/tv/mfslib.tcl

# RetryTransaction without debug output
proc MyRetryTransaction { body } {
    global errorInfo errorCode

    while { 1 } {
        set code [catch {transaction {uplevel $body}} string]
        if { $code == 0 } {
            return $string
        } elseif { $code == 1 } {
            if { $errorCode == "errTmActiveLockConflict" ||
                 $errorCode == "errTmBackgroundHoldoff" ||
                 $errorCode == "errFsLockConflict" } {
                after 100
                # retry the transaction
                continue
            }
            return -code error -errorinfo $errorInfo -errorcode $errorCode $string
        } elseif { $code == 2 } {
            return -code return $string
        } elseif { $code == 3 } {
            return -code break $string
        } elseif { $code == 4 } {
            return -code continue $string
        }
    }
}

proc MyForeachMfsFile { idVar nameVar typeVar dirName prefix body } {
    global errorInfo errorCode

    upvar $idVar   id
    upvar $nameVar name
    upvar $typeVar type

    # Get the first batch of names
    MyRetryTransaction {
        if { [catch {mfs scan $dirName -start $prefix -count 50} batch] } {
            global errorCode errorInfo
            if { $errorCode == "errNmNameNotFound" } {
                return
            } else {
                error $batch $errorInfo $errorCode
            }
        }
    }

    while { [llength $batch] > 0 } {
        # Execute the body for each item in this batch
        foreach item $batch {
            set id   [lindex $item 0]
            set name [lindex $item 1]
            set type [lindex $item 2]

            # bail if we're past the entries that start with the given prefix
            if { ! [PrefixMatches $prefix $name] } {
                return
            }

            set code [catch {uplevel $body} string]
            if { $code == 1 } {
                return -code error -errorinfo $errorInfo \
                        -errorcode $errorCode $string
            } elseif { $code == 3 } {
                # this is a break in the body.  just return normally
                return
            } elseif { $code != 0 } {
                return -code $code $string
            }
        }

        # Get the next batch
        set lastName [lindex [lindex $batch end] 1]
        MyRetryTransaction {
            set batch [mfs scan $dirName -start $lastName -count 50]
            if { $lastName == [lindex [lindex $batch 0] 1] } {
                set batch [lrange $batch 1 end]
            }
        }
    }
}



#
# Main program
#

puts ""
puts "noppv tcl script:"
puts "  Removes PPV flag from existing shows in the Tivo database."
puts ""

# Open the database
set db [dbopen]
set starttime [clock seconds]
set path [file dirname $argv0]
set path "$path/st.dat"

# Get timezone from command line (default central -6)
set dump ""
set tz [lindex $argv 0]
if { $tz == "" } {
	set tz -6
} elseif { $tz == "dump" } {
	set dump "true"
	puts "** Dumping stations to file: $path **"
	puts ""
	set inp [open $path "w"]
	close $inp
	set tz -6
}

if { $dump != "true" } {
	puts "** Timezone set to GMT $tz **"
	puts ""
}

# See if we have a st.dat file
set load ""
if { [file exists $path] && $dump != "true" } {
	set load "file"
	puts "** Found file '$path' **"
	puts "** Loading stations from file. **"
	puts ""
}

if { $load != "file" } {
	# Open our channel list
	MyRetryTransaction {
		set obj [db $db open /Setup]
		set s [dbobj $obj get Source]
		set channels [dbobj $s gettarget Channel]
	}
} else {
	set inp [open $path "r"]
	while { [eof $inp] == 0 } {
		set channel [gets $inp]
		set c1 [string range $channel 0 0]
		if { $channel == "" || $c1 == " " || $c1 == "#" } {
			continue
		}
		lappend channels $channel
	}
	close $inp
}

# Get station ids of PPV channels
set showcount 0
foreach channel $channels {
	if { $load != "file" } {
		MyRetryTransaction {
			regexp {([0-9]*)/(.*)} $channel junk fsid subobjid
			set obj [db $db openidconstruction $fsid $subobjid]
			set num [dbobj $obj get Number]
			set stationfsid [dbobj $obj gettarget Station]
			set name [dbobj [dbobj $obj get ApgChannel] get ShortName]
		}
	} else {
	regexp {([0-9]*):(.*):([0-9]*)} $channel junk num name sid
	}

	# Skip these channels
	if { $name == "TIVOD" || $name == "TIVOR" } {
		continue
	}

	# Only process PPV stations (101 - 199, 594 - 598)
	if { ($num >= 101 && $num < 200) || ($num >= 594 && $num <= 598) } {
		if { $load != "file" } {
			regexp {([0-9]*)/(.*)} $stationfsid junk sid sobjid
		}

		puts "Channel number $num"
		if { $dump == "true" } {
			if { [catch {set inp [open $path "a"]}] == 0 } {
				puts $inp "$num:$name:$sid"
				close $inp
				continue
			}
		}

		# Find all schedules for this station
		MyForeachMfsFile fsid name type "/Schedule" "$sid:" {
			MyRetryTransaction {
				# Open StationDay listing
				if {[catch {set sday [db $db openid $fsid]}] != 0} {
					puts "  Showing has expired; skipping. (fsid $fsid)"
					continue
				}

				# Get showings for this day
				set showings [dbobj $sday get Showing]
				foreach show $showings {
					# See if this showing has the PPV flag
					if { [dbobj $show gettarget ApgScheduleEvent] != "" } {
						# Get date and time
						set days [expr [dbobj $show get Date] - 11719]
						set seconds [dbobj $show get Time]
						set date [clock scan "02/01/2002 $days days $seconds seconds $tz hours"]

						# Get program info
						set program [dbobj $show get Program]
						set title [dbobj $program get Title]

						# Remove the ppv flag
						dbobj $show remove ApgScheduleEvent
						puts "  Fixed: '[clock format $date -format "%D %I:%M %p" -gmt false]' $title"
						incr showcount 1
					}
				}
			}
		}
	}
}

puts ""
puts "Fixed $showcount showings."
set endtime [clock seconds]
puts "Process took [expr $endtime - $starttime] seconds."

dbclose $db
